#ifndef CSTATEDIALOG_H
#define CSTATEDIALOG_H
#include <gtkmm.h>
#include "CGtkTextView.h"
#include <iostream>
#include <map>

// Presents a window allowing the user to capture a "snapshot" of the
// currently installed packages, and also provides a method of restoring
// that snapshot.
// See COPYING for license information.

class CStateDialog : public Gtk::Dialog
{
public:
	CStateDialog(packagemap_t& packages);
protected:
	Glib::ustring	selectfile();
	void		savestate(packagemap_t& packages);
	void		restorestate(packagemap_t& packages);

	Gtk::Frame	_Frame;
	CGtkTextView	_Text;
};

CStateDialog::CStateDialog(packagemap_t& packages)
{
	// Set up the UI.
	set_title("Capture/Restore State");
	set_border_width(6);
	set_default_size(600, 450);
	add_button("Capture state", 1);
	add_button("Restore state", 2);
	add_button("View state", 3);
	add_button(Gtk::Stock::CLOSE, -4);
	set_default_response(-4);
	get_vbox()->pack_start(_Frame);
	_Frame.add(_Text);
	show_all_children();
	_Text.SetText("This dialog allows you to save the list of currently installed packages using the"
		       "\n\"Capture state\" button, or restore a previously saved state using the"
		       "\n\"Restore state\" button.  This feature is not intended as a means to back up"
		       "\nand restore the files on your computer.  Instead, it is for use in situations"
		       "\nwhere you might want to test out some new packages and then remove all of those"
		       "\npackages when you are finished.  It is especially useful when you want to try"
		       "\na major change.  For example, you can save the state of your machine, install a"
		       "\nnew window manager, and restore the state when you decide you don't want that"
		       "\nwindow manager installed any more.");

	// Continually call run() until it's time to exit.
	bool	_end = false;
	while ( !_end ) {
		switch ( run() ) {
		case 1:	// Save the current state
			savestate(packages);
			break;
		case 2:	// Restore a previously saved state
			restorestate(packages);
			break;
		case 3:	// Show the contents of a state file
			_Text.GetTextFromFile(selectfile());
			break;
		case -4:
			_end = true;
			break;
		}
	}
}

Glib::ustring CStateDialog::selectfile()
{	// Returns a filename to be opened (not saved).
	Gtk::FileChooserDialog	_fdlg("Choose a file", Gtk::FILE_CHOOSER_ACTION_OPEN);
	_fdlg.set_transient_for(*this);
	_fdlg.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	_fdlg.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);
	_fdlg.set_current_folder("/usr/share/zendo/states");

	if ( _fdlg.run() != Gtk::RESPONSE_OK )
		return Glib::ustring("");
	return _fdlg.get_filename();
}

void CStateDialog::savestate(packagemap_t& packages)
{	// Get a filename to save it to, then save it.
	Gtk::FileChooserDialog	_fdlg("Choose a file", Gtk::FILE_CHOOSER_ACTION_SAVE);
	_fdlg.set_transient_for(*this);
	_fdlg.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	_fdlg.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);
	_fdlg.set_current_folder("/usr/share/zendo/states");
	if ( _fdlg.run() != Gtk::RESPONSE_OK )
		return;

	std::ofstream	_ofile;
	_ofile.open(_fdlg.get_filename().c_str(), std::ios_base::out);
	for ( packagemap_t::iterator _iter = packages.begin(); _iter != packages.end(); ++_iter ) {
		if ( _iter->second.installedversion.length() )
			_ofile << _iter->second.name << "-" << _iter->second.installedversion << "\n";
	}
	_ofile.close();
	_Text.SetText("Current state has been saved to: " + _fdlg.get_filename());
}

void CStateDialog::restorestate(packagemap_t& packages)
{	// Read the state from a file, and set the install/remove status of packages accordingly.
	Glib::ustring	_fname = selectfile();
	if ( !_fname.length() ) return;
	std::ifstream	_ifile;
	Glib::ustring	_summary;
	std::map<Glib::ustring, Glib::ustring>	_statemap;

	_ifile.open(_fname.c_str(), std::ios_base::in);
	while ( _ifile.is_open() && !_ifile.eof() ) {
		std::string	_input;
		getline(_ifile, _input);
		if ( !_input.length() ) break;

		// Seperate the name and version, and create the db.
		int	bpos = _input.length();
		for ( int i = 0; i < 3; i++ )
			bpos = _input.rfind('-', bpos - 1);
		_statemap[_input.substr(0, bpos)] = _input.substr(bpos + 1);
	}
	_ifile.close();

	_summary = "State has been restored from: " + _fname +
		"\n\nThe appropriate packages have been marked for installation,"
		"\nupdate, and removal.  To apply the changes, just exit this"
		"\nwindow and click the apply button\n";

	// Make the changes to the pending installations/removals based
	// on the state the user has opened.
	for ( packagemap_t::iterator _iter = packages.begin(); _iter != packages.end(); ++_iter ) {
		if ( _iter->second.installedversion.length() &&
				(_statemap.find(_iter->second.name) == _statemap.end()) &&
				(_iter->second.color != COLOR_BLACKLIST) ) {
			_iter->second.color = COLOR_REMOVE;
			_summary += "\nRemove: " + _iter->second.name;
			_statemap[_iter->second.name] = "";
		} else if ( !_iter->second.installedversion.length() &&
				(_statemap.find(_iter->second.name) != _statemap.end()) &&
				(_iter->second.color != COLOR_BLACKLIST) ) {
			_iter->second.color = COLOR_INSTALL;
			_summary += "\nInstall: " + _iter->second.name;
			_statemap[_iter->second.name] = "";
		} else if ( _iter->second.installedversion.length() &&	// Upgrades/downgrades
				(_statemap.find(_iter->second.name) != _statemap.end()) &&
				(_iter->second.installedversion != _statemap[_iter->second.name]) &&
				(_iter->second.mirrorversion == _statemap[_iter->second.name]) &&
				(_iter->second.color != COLOR_BLACKLIST) ) {
			_iter->second.color = COLOR_INSTALL;
			_summary += "\nUpdate: " + _iter->second.name;
			_statemap[_iter->second.name] = "";
		}
		if ( _statemap.find(_iter->second.name) != _statemap.end() &&
				((_statemap[_iter->second.name].length() &&
				_statemap[_iter->second.name] == _iter->second.installedversion) ||
				_iter->second.color == COLOR_BLACKLIST) )
			_statemap[_iter->second.name] = "";
	}

	// Check for packages from the statemap that were not processed.
	bool	_shown = false;
	for ( std::map<Glib::ustring, Glib::ustring>::iterator _iter = _statemap.begin(); _iter != _statemap.end(); ++_iter ) {
		if ( _iter->second.length() ) {
			if ( !_shown ) _summary += "\n\nThe following packages do not exist on the mirror:\n";
			_summary += "\n" + _iter->first + "-" + _iter->second;
			_shown = true;
		}
	}
	_Text.SetText(_summary);
}

#endif
